package com.webonlineapp.fulldemo;

import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Base64;
import java.util.Scanner;

import org.apache.commons.io.FileUtils;

import com.hyjavacharts.hyjavaimages.Chrome;
import com.hyjavacharts.hyjavaimages.HtmlOptions;
import com.hyjavacharts.hyjavaimages.ImageException;
import com.hyjavacharts.hyjavaimages.ImageFormat;
import com.hyjavacharts.hyjavaimages.ImageGenerator;
import com.hyjavacharts.hyjavaimages.ImageOptions;
import com.hyjavacharts.hyjavaimages.JavaScriptOptions;
import com.hyjavacharts.hyjavaimages.LibraryType;
import com.hyjavacharts.hyjavaimages.PdfOptions;

/**
 * This is an example application showing usage of the HyJavaImages library.
 * To build/run this example you would need to purchase a HyJavaImages license.
 * 
 * This example needs a directory to store the created images. See the imagesPath variable below.
 * 
 */
public class FullDemo {
	String imagesPath = "c:/var/chart-images/";		// You will need to create this directory
	
	
	/**
	 * Run the examples.
	 * 
	 */
	public void run() {
		// Startup Chrome headless
		Chrome chrome = Chrome.getInstance();
		// If you wish to run "old" chrome headless, uncomment the line below. 
		// The chrome developers will remove "old" chrome headless in Chrome 132 (January 2025).
		//chrome.setChromeArguments(new ChromeArguments().setChromeHeadlessType(ChromeHeadlessType.HEADLESS_OLD));
		chrome.start();
		
		// Generate as many charts as needed
		try {
			exampleTests();
		} catch (ImageException e) {
			e.printStackTrace();
		}

		// Close down Chrome headless
        chrome.stop();
	}
	
	
	/**
	 * Create images of various charts.
	 * 
	 * @throws ImageException
	 */
	private void exampleTests() throws ImageException {
		System.out.println("Demo Started");
		
		String base64;
		ImageGenerator img;
		
		// Highchart line example from chart options
		img = new ImageGenerator();
		img.setJavaScriptLibrary(LibraryType.HIGHCHARTS, "12.1.2", "highcharts.js");
		base64 = img.toBase64Image(getDemoChartResource("highcharts-line-chart-options.txt"));
		createImageFile("highchart-line" + ".png", base64);
		
		// Highchart pie example from chart options
		img = new ImageGenerator();
		img.setJavaScriptLibrary(LibraryType.HIGHCHARTS, "12.1.2", "highcharts.js");
		base64 = img.toBase64Image(getDemoChartResource("highcharts-pie-chart-options.txt"));
		createImageFile("highchart-pie.png", base64);
		
		// Highchart pie example from chart options - JPEG
		img = new ImageGenerator();
		img.setJavaScriptLibrary(LibraryType.HIGHCHARTS, "12.1.2", "highcharts.js");
		img.setImageOptions(new ImageOptions().setFormat(ImageFormat.JPEG).setQuality(100));
		base64 = img.toBase64Image(getDemoChartResource("highcharts-pie-chart-options.txt"));
		createImageFile("highchart-pie.jpeg", base64);
		
		// Highchart pie example from chart options - resolution x 4
		img = new ImageGenerator();
		img.setJavaScriptLibrary(LibraryType.HIGHCHARTS, "12.1.2", "highcharts.js");
		img.setImageOptions(new ImageOptions().setScale(2));
		base64 = img.toBase64Image(getDemoChartResource("highcharts-pie-chart-options.txt"));
		createImageFile("highchart-pie-x4.png", base64);
		
		// Highchart pie example from chart options - pdf
		img = new ImageGenerator();
		img.setJavaScriptLibrary(LibraryType.HIGHCHARTS, "12.1.2", "highcharts.js");
		img.setPdfOptions(new PdfOptions().setLandscape(true));
		base64 = img.toBase64Pdf(getDemoChartResource("highcharts-pie-chart-options.txt"));
		createImageFile("highchart-pie.pdf", base64);
		
		// Highstock column example from chart options
		img = new ImageGenerator();
		img.setJavaScriptLibrary(LibraryType.HIGHCHARTS_STOCK, "12.1.2", "stock/highstock.js");
		base64 = img.toBase64Image(getDemoChartResource("highstock-chart-options.txt"));
		createImageFile("highstock-column.png", base64);
		
		// ApexCharts column example from chart options
		img = new ImageGenerator();
		img.setJavaScriptLibrary(LibraryType.APEXCHARTS, "4.3.0");
		base64 = img.toBase64Image(getDemoChartResource("apex-chart-options.txt"));
		createImageFile("apexcharts-column.png", base64);
		
		// HTML for e-charts example from html text
		img = new ImageGenerator();
		base64 = img.htmlToBase64Image(getDemoChartResource("echart-html.txt"));
		createImageFile("echart.png", base64);
		
		// HTML for chartjs example from html text
		img = new ImageGenerator();
		base64 = img.htmlToBase64Image(getDemoChartResource("chartjs-html.txt"));
		createImageFile("chartjs.png", base64);
		
		// URL example
		img = new ImageGenerator();
		URL url = null;
		try {
			url = new URL("https://www.google.com");
		} catch (MalformedURLException e1) {
			e1.printStackTrace();
		}
		
		base64 = img.toBase64Image(url);
		createImageFile("url.png", base64);
		
		// Highcharts line and pie from chart options in 1 image example
		img = new ImageGenerator();
		img.setJavaScriptLibrary(LibraryType.HIGHCHARTS, "12.1.2", "highcharts.js");
		img.setHtmlOptions(new HtmlOptions().setChartDivSize("300px", "300px"));	// sets div height for both charts
		base64 = img.toBase64Image(getDemoChartResource("highcharts-line-chart-options.txt"),
								   getDemoChartResource("highcharts-pie-chart-options.txt"));
		createImageFile("highchart-2-charts.png", base64);
		
		// Vaadin charts example
		img = new ImageGenerator();
		img.setJavaScriptLibrary(LibraryType.HIGHCHARTS, "12.1.2", "highcharts.js");
		img.setJavaScriptOptions(new JavaScriptOptions()
					.setJavaScriptLibraryAdditional("https://www.hyjavacharts.com/content/js/vaadin-inflate.js")
					.setJavaScriptChartOptions("__inflateFunctions(chartOptions);"));
		base64 = img.toBase64Image(getDemoChartResource("vaadin-chart-options.txt"));
		createImageFile("vaadin-charts.png", base64);
		
		System.out.println("Demo Completed");
	}
	

	
	/**
	 * Create the image file from the base64 encoded string.
	 * 
	 * @param fileName
	 * @param base64
	 */
	private void createImageFile(String fileName, String base64) {
		byte [] imageBytes = Base64.getDecoder().decode(base64);
		
		File imageFile = new File(imagesPath + fileName);
		try {
			FileUtils.writeByteArrayToFile(imageFile, imageBytes);
			System.out.println(imagesPath + fileName + " created");
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * Get demo chart resources. Either chart options or html text.
	 * 
	 * @param chartResource
	 * @return
	 */
	private String getDemoChartResource(String chartResource) {
		Scanner scanner = new Scanner(FullDemo.class.getResourceAsStream(chartResource), "UTF-8");
		String text = scanner.useDelimiter("\\A").next();
		scanner.close();
		
		return text;
	}
	
}
